@@ -112,7 +112,6 @@ verify_certificate (gnutls_session session, const char *hostname, GError **err)
 			session, &cert_list_size);
 
 		if (cert_list == NULL) {
-			gnutls_x509_crt_deinit (cert);
 			g_set_error (err, SOUP_SSL_ERROR,
 				     SOUP_SSL_ERROR_CERTIFICATE,
 				     "No SSL certificate was found.");
@@ -121,7 +120,6 @@ verify_certificate (gnutls_session session, const char *hostname, GError **err)
 
 		if (gnutls_x509_crt_import (cert, &cert_list[0],
 					    GNUTLS_X509_FMT_DER) < 0) {
-			gnutls_x509_crt_deinit (cert);
 			g_set_error (err, SOUP_SSL_ERROR,
 				     SOUP_SSL_ERROR_CERTIFICATE,
 				     "The SSL certificate could not be parsed.");
@@ -129,14 +127,11 @@ verify_certificate (gnutls_session session, const char *hostname, GError **err)
 		}
 
 		if (!gnutls_x509_crt_check_hostname (cert, hostname)) {
-			gnutls_x509_crt_deinit (cert);
 			g_set_error (err, SOUP_SSL_ERROR,
 				     SOUP_SSL_ERROR_CERTIFICATE,
 				     "The SSL certificate does not match the hostname.");
 			return FALSE;
 		}
-
-		gnutls_x509_crt_deinit (cert);
 	}
 
 	return TRUE;
@@ -163,15 +158,12 @@ again:
 	}
 
 	if (result < 0) {
-		g_set_error (err, SOUP_SSL_ERROR,
-			     SOUP_SSL_ERROR_HANDSHAKE_FAILED,
-			     "SSL handshake failed: %s",
-			     gnutls_strerror (result));
+		g_set_error (err, G_IO_CHANNEL_ERROR,
+			     G_IO_CHANNEL_ERROR_FAILED,
+			     "Unable to handshake");
 		return G_IO_STATUS_ERROR;
 	}
 
-	chan->established = TRUE;
-
 	if (chan->type == SOUP_SSL_TYPE_CLIENT && chan->creds->have_ca_file &&
 	    !verify_certificate (chan->session, chan->hostname, err))
 		return G_IO_STATUS_ERROR;
@@ -198,6 +190,8 @@ again:
 		if (result == G_IO_STATUS_AGAIN ||
 		    result == G_IO_STATUS_ERROR)
 			return result;
+
+		chan->established = TRUE;
 	}
 
 	result = gnutls_record_recv (chan->session, buf, count);
@@ -258,6 +252,8 @@ again:
 		if (result == G_IO_STATUS_AGAIN ||
 		    result == G_IO_STATUS_ERROR)
 			return result;
+
+		chan->established = TRUE;
 	}
 
 	result = gnutls_record_send (chan->session, buf, count);
@@ -394,22 +390,8 @@ soup_gnutls_pull_func (gnutls_transport_ptr_t transport_data,
 	SoupGNUTLSChannel *chan = transport_data;
 	ssize_t nread;
 
-	nread = recv (chan->sockfd, buf, buflen, 0);
-#ifdef G_OS_WIN32
-	{
-		int wsa_errno = WSAGetLastError ();
-		chan->eagain = (nread == SOCKET_ERROR && (wsa_errno == WSAEWOULDBLOCK ||
-							  wsa_errno == WSAETIMEDOUT));
-		if (nread == SOCKET_ERROR)
-			gnutls_transport_set_errno (chan->session,
-						    ((wsa_errno == WSAEWOULDBLOCK ||
-						      wsa_errno == WSAETIMEDOUT) ? EAGAIN :
-						     (wsa_errno == WSAEINTR ? EINTR :
-						      EIO)));
-	}
-#else
+	nread = read (chan->sockfd, buf, buflen);
 	chan->eagain = (nread == -1 && errno == EAGAIN);
-#endif
 	return nread;
 }
 
@@ -420,20 +402,8 @@ soup_gnutls_push_func (gnutls_transport_ptr_t transport_data,
 	SoupGNUTLSChannel *chan = transport_data;
 	ssize_t nwrote;
 
-	nwrote = send (chan->sockfd, buf, buflen, 0);
-#ifdef G_OS_WIN32
-	{
-		int wsa_errno = WSAGetLastError ();
-		chan->eagain = (nwrote == SOCKET_ERROR && wsa_errno == WSAEWOULDBLOCK);
-		if (nwrote == SOCKET_ERROR)
-			gnutls_transport_set_errno (chan->session,
-						    (wsa_errno == WSAEWOULDBLOCK ? EAGAIN :
-						     (wsa_errno == WSAEINTR ? EINTR :
-						      EIO)));
-	}
-#else
+	nwrote = write (chan->sockfd, buf, buflen);
 	chan->eagain = (nwrote == -1 && errno == EAGAIN);
-#endif
 	return nwrote;
 }
 
@@ -486,11 +456,6 @@ soup_ssl_wrap_iochannel (GIOChannel *sock, gboolean non_blocking,
 
 	if (type == SOUP_SSL_TYPE_SERVER)
 		gnutls_dh_set_prime_bits (session, DH_BITS);
-	else {
-		// gnutls defaults to requiring at least 768-bit keys,
-		// but there are some lame servers out there...
-		gnutls_dh_set_prime_bits (session, 256);
-	}
 
 	chan = g_slice_new0 (SoupGNUTLSChannel);
 	chan->real_sock = sock;
@@ -524,54 +489,6 @@ soup_ssl_wrap_iochannel (GIOChannel *sock, gboolean non_blocking,
 GCRY_THREAD_OPTION_PTHREAD_IMPL;
 #endif
 
-#ifdef G_OS_WIN32
-
-static int
-soup_gcry_win32_mutex_init (void **priv)
-{
-	int err = 0;
-	CRITICAL_SECTION *lock = (CRITICAL_SECTION*)malloc (sizeof (CRITICAL_SECTION));
-
-	if (!lock)
-		err = ENOMEM;
-	if (!err) {
-		InitializeCriticalSection (lock);
-		*priv = lock;
-	}
-	return err;
-}
-
-static int
-soup_gcry_win32_mutex_destroy (void **lock)
-{
-	DeleteCriticalSection ((CRITICAL_SECTION*)*lock);
-	free (*lock);
-	return 0;
-}
-
-static int
-soup_gcry_win32_mutex_lock (void **lock)
-{
-	EnterCriticalSection ((CRITICAL_SECTION*)*lock);
-	return 0;
-}
-
-static int
-soup_gcry_win32_mutex_unlock (void **lock)
-{
-	LeaveCriticalSection ((CRITICAL_SECTION*)*lock);
-	return 0;
-}
-
-
-static struct gcry_thread_cbs soup_gcry_threads_win32 = {		 \
-	(GCRY_THREAD_OPTION_USER | (GCRY_THREAD_OPTION_VERSION << 8)),	 \
-	NULL, soup_gcry_win32_mutex_init, soup_gcry_win32_mutex_destroy, \
-	soup_gcry_win32_mutex_lock, soup_gcry_win32_mutex_unlock,	 \
-	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
-
-#endif
-
 static void
 soup_gnutls_init (void)
 {
@@ -580,8 +497,6 @@ soup_gnutls_init (void)
 	if (g_once_init_enter (&inited_gnutls)) {
 #if defined(GCRY_THREAD_OPTION_PTHREAD_IMPL) && !defined(G_OS_WIN32)
 		gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
-#elif defined(G_OS_WIN32)
-		gcry_control (GCRYCTL_SET_THREAD_CBS, &soup_gcry_threads_win32);
 #endif
 		gnutls_global_init ();
 		g_once_init_leave (&inited_gnutls, TRUE);
@@ -614,10 +529,6 @@ soup_ssl_get_client_credentials (const char *ca_file)
 	creds = g_slice_new0 (SoupSSLCredentials);
 	gnutls_certificate_allocate_credentials (&creds->creds);
 
-	/* http://bugzilla.gnome.org/show_bug.cgi?id=589323 */
-	gnutls_certificate_set_verify_flags (creds->creds,
-					     GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT);
-
 	if (ca_file) {
 		creds->have_ca_file = TRUE;
 		status = gnutls_certificate_set_x509_trust_file (
