@@ -12,7 +12,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "soup-coding.h"
 #include "soup-connection.h"
 #include "soup-message.h"
 #include "soup-message-private.h"
@@ -54,9 +53,6 @@ typedef struct {
 	SoupMessageBody      *read_body;
 	goffset               read_length;
 
-	gboolean              need_content_sniffed, need_got_chunk;
-	SoupMessageBody      *sniff_data;
-
 	SoupMessageIOState    write_state;
 	SoupEncoding          write_encoding;
 	GString              *write_buf;
@@ -109,9 +105,6 @@ soup_message_io_cleanup (SoupMessage *msg)
 	if (io->write_chunk)
 		soup_buffer_free (io->write_chunk);
 
-	if (io->sniff_data)
-		soup_message_body_free (io->sniff_data);
-
 	g_slice_free (SoupMessageIOData, io);
 }
 
@@ -180,34 +173,17 @@ soup_message_io_finished (SoupMessage *msg)
 
 static void io_read (SoupSocket *sock, SoupMessage *msg);
 
-static gboolean
-request_is_idempotent (SoupMessage *msg)
-{
-	/* FIXME */
-	return (msg->method == SOUP_METHOD_GET);
-}
-
 static void
 io_error (SoupSocket *sock, SoupMessage *msg, GError *error)
 {
-	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
-	SoupMessageIOData *io = priv->io_data;
-
-	if (error && error->domain == SOUP_SSL_ERROR) {
-		soup_message_set_status_full (msg,
-					      SOUP_STATUS_SSL_FAILED,
-					      error->message);
-	} else if (io->mode == SOUP_MESSAGE_IO_CLIENT &&
-		   io->read_state <= SOUP_MESSAGE_IO_STATE_HEADERS &&
-		   io->read_meta_buf->len == 0 &&
-		   soup_connection_get_ever_used (io->conn) &&
-		   !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT) &&
-		   request_is_idempotent (msg)) {
-		/* Connection got closed, but we can safely try again */
-		priv->io_status = SOUP_MESSAGE_IO_STATUS_QUEUED;
-	} else if (!SOUP_STATUS_IS_TRANSPORT_ERROR (msg->status_code))
-		soup_message_set_status (msg, SOUP_STATUS_IO_ERROR);
-
+	if (!SOUP_STATUS_IS_TRANSPORT_ERROR (msg->status_code)) {
+		if (error && error->domain == SOUP_SSL_ERROR) {
+			soup_message_set_status_full (msg,
+						      SOUP_STATUS_SSL_FAILED,
+						      error->message);
+		} else
+			soup_message_set_status (msg, SOUP_STATUS_IO_ERROR);
+	}
 	if (error)
 		g_error_free (error);
 
@@ -231,55 +207,6 @@ io_disconnected (SoupSocket *sock, SoupMessage *msg)
 	io_error (sock, msg, NULL);
 }
 
-static gboolean
-io_handle_sniffing (SoupMessage *msg, gboolean done_reading)
-{
-	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
-	SoupMessageIOData *io = priv->io_data;
-	SoupBuffer *sniffed_buffer;
-	char *sniffed_mime_type;
-	GHashTable *params = NULL;
-
-	if (!priv->sniffer)
-		return TRUE;
-
-	if (!io->sniff_data) {
-		io->sniff_data = soup_message_body_new ();
-		io->need_content_sniffed = TRUE;
-	}
-
-	if (io->need_content_sniffed) {
-		if (io->sniff_data->length < priv->bytes_for_sniffing &&
-		    !done_reading)
-			return TRUE;
-
-		io->need_content_sniffed = FALSE;
-		sniffed_buffer = soup_message_body_flatten (io->sniff_data);
-		sniffed_mime_type = soup_content_sniffer_sniff (priv->sniffer, msg, sniffed_buffer, &params);
-
-		SOUP_MESSAGE_IO_PREPARE_FOR_CALLBACK;
-		soup_message_content_sniffed (msg, sniffed_mime_type, params);
-		g_free (sniffed_mime_type);
-		if (params)
-			g_hash_table_destroy (params);
-		if (sniffed_buffer)
-			soup_buffer_free (sniffed_buffer);
-		SOUP_MESSAGE_IO_RETURN_VAL_IF_CANCELLED_OR_PAUSED (FALSE);
-	}
-
-	if (io->need_got_chunk) {
-		io->need_got_chunk = FALSE;
-		sniffed_buffer = soup_message_body_flatten (io->sniff_data);
-
-		SOUP_MESSAGE_IO_PREPARE_FOR_CALLBACK;
-		soup_message_got_chunk (msg, sniffed_buffer);
-		soup_buffer_free (sniffed_buffer);
-		SOUP_MESSAGE_IO_RETURN_VAL_IF_CANCELLED_OR_PAUSED (FALSE);
-	}
-
-	return TRUE;
-}
-
 /* Reads data from io->sock into io->read_meta_buf. If @to_blank is
  * %TRUE, it reads up until a blank line ("CRLF CRLF" or "LF LF").
  * Otherwise, it reads up until a single CRLF or LF.
@@ -343,54 +270,9 @@ read_metadata (SoupMessage *msg, gboolean to_blank)
 		}
 	}
 
-	if (soup_socket_is_ssl (io->sock)) {
-		gboolean trusted_certificate;
-
-		g_object_get (io->sock,
-			      SOUP_SOCKET_TRUSTED_CERTIFICATE, &trusted_certificate,
-			      NULL);
-
-		if (trusted_certificate)
-			soup_message_set_flags (msg, priv->msg_flags | SOUP_MESSAGE_CERTIFICATE_TRUSTED);
-	}
-
 	return TRUE;
 }
 
-static SoupBuffer *
-content_decode (SoupMessage *msg, SoupBuffer *buf)
-{
-	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
-	SoupCoding *decoder;
-	SoupBuffer *decoded;
-	GError *error = NULL;
-	GSList *d;
-
-	for (d = priv->decoders; d; d = d->next) {
-		decoder = d->data;
-
-		decoded = soup_coding_apply (decoder, buf->data, buf->length,
-					     FALSE, &error);
-		if (error) {
-			if (g_error_matches (error, SOUP_CODING_ERROR, SOUP_CODING_ERROR_INTERNAL_ERROR))
-				g_warning ("Content-Decoding error: %s\n", error->message);
-			g_error_free (error);
-
-			soup_message_set_flags (msg, priv->msg_flags & ~SOUP_MESSAGE_CONTENT_DECODED);
-			break;
-		}
-		if (buf)
-			soup_buffer_free (buf);
-
-		if (decoded)
-			buf = decoded;
-		else
-			return NULL;
-	}
-
-	return buf;
-}
-
 /* Reads as much message body data as is available on io->sock (but no
  * further than the end of the current message body or chunk). On a
  * successful read, emits "got_chunk" (possibly multiple times), and
@@ -412,9 +294,6 @@ read_body_chunk (SoupMessage *msg)
 	GError *error = NULL;
 	SoupBuffer *buffer;
 
-	if (!io_handle_sniffing (msg, FALSE))
-		return FALSE;
-
 	while (read_to_eof || io->read_length > 0) {
 		if (priv->chunk_allocator) {
 			buffer = priv->chunk_allocator (msg, io->read_length, priv->chunk_allocator_data);
@@ -441,22 +320,9 @@ read_body_chunk (SoupMessage *msg)
 
 		if (status == SOUP_SOCKET_OK && nread) {
 			buffer->length = nread;
-			io->read_length -= nread;
-
-			buffer = content_decode (msg, buffer);
-			if (!buffer)
-				continue;
-
 			soup_message_body_got_chunk (io->read_body, buffer);
 
-			if (io->need_content_sniffed) {
-				soup_message_body_append_buffer (io->sniff_data, buffer);
-				soup_buffer_free (buffer);
-				io->need_got_chunk = TRUE;
-				if (!io_handle_sniffing (msg, FALSE))
-					return FALSE;
-				continue;
-			}
+			io->read_length -= nread;
 
 			SOUP_MESSAGE_IO_PREPARE_FOR_CALLBACK;
 			soup_message_got_chunk (msg, buffer);
@@ -700,8 +566,7 @@ io_write (SoupSocket *sock, SoupMessage *msg)
 				 io->write_chunk->length, TRUE))
 			return;
 
-		if (io->mode == SOUP_MESSAGE_IO_SERVER)
-			soup_message_body_wrote_chunk (io->write_body, io->write_chunk);
+		soup_message_body_wrote_chunk (io->write_body, io->write_chunk);
 		io->write_body_offset += io->write_chunk->length;
 		soup_buffer_free (io->write_chunk);
 		io->write_chunk = NULL;
@@ -744,8 +609,7 @@ io_write (SoupSocket *sock, SoupMessage *msg)
 				 io->write_chunk->length, TRUE))
 			return;
 
-		if (io->mode == SOUP_MESSAGE_IO_SERVER)
-			soup_message_body_wrote_chunk (io->write_body, io->write_chunk);
+		soup_message_body_wrote_chunk (io->write_body, io->write_chunk);
 		soup_buffer_free (io->write_chunk);
 		io->write_chunk = NULL;
 
@@ -918,21 +782,6 @@ io_read (SoupSocket *sock, SoupMessage *msg)
 			return;
 
 	got_body:
-		if (!io_handle_sniffing (msg, TRUE)) {
-			/* If the message was paused (as opposed to
-			 * cancelled), we need to make sure we wind up
-			 * back here when it's unpaused, even if it
-			 * was doing a chunked or EOF-terminated read
-			 * before.
-			 */
-			if (io == priv->io_data) {
-				io->read_state = SOUP_MESSAGE_IO_STATE_BODY;
-				io->read_encoding = SOUP_ENCODING_CONTENT_LENGTH;
-				io->read_length = 0;
-			}
-			return;
-		}
-
 		io->read_state = SOUP_MESSAGE_IO_STATE_FINISHING;
 
 		SOUP_MESSAGE_IO_PREPARE_FOR_CALLBACK;
